#!/bin/bash

# Better signal handling - forward signals to child processes
trap 'kill -TERM $SERVER_PID; kill -TERM $CADDY_PID; exit' TERM INT

set -eu

echo "==> Starting Ente Cloudron app..."

# Create necessary directories
mkdir -p /app/data/config /app/data/storage /app/data/caddy /app/data/go /app/data/logs

# Add comment about Cloudron filesystem limitations
echo "==> NOTE: Running in Cloudron environment with limited write access"
echo "==> Writable directories: /app/data, /tmp, /run"

# Define the server directory
SERVER_DIR="/app/code/server"
if [ ! -d "$SERVER_DIR" ]; then
    if [ -d "/app/code/museum" ]; then
        SERVER_DIR="/app/code/museum"
    else
        # Look for main.go in likely places
        SERVER_DIR=$(dirname $(find /app/code -name "main.go" -path "*/server*" -o -path "*/museum*" | head -1))
        if [ ! -d "$SERVER_DIR" ]; then
            echo "==> WARNING: Could not find server directory, using /app/code as fallback"
            SERVER_DIR="/app/code"
        fi
    fi
fi
echo "==> Using server directory: $SERVER_DIR"

# One-time initialization tracking
if [[ ! -f /app/data/.initialized ]]; then
    echo "==> Fresh installation, setting up data directory..."
    
    echo "==> DEBUG: Full repository structure at /app/code"
    find /app/code -type d -maxdepth 3 -not -path "*/node_modules/*" -not -path "*/\.*" | sort

    echo "==> DEBUG: Looking for Go files"
    find /app/code -name "*.go" | grep -v test | sort | head -10

    echo "==> DEBUG: Looking for server-related directories"
    find /app/code -type d -path "*/server*" -o -path "*/museum*" | sort

    echo "==> DEBUG: All package.json files in repository"
    find /app/code -name "package.json" -not -path "*/node_modules/*" | sort

    echo "==> DEBUG: Looking for web app directories"
    find /app/code -type d -path "*/web*" | sort

    echo "==> DEBUG: Web app directories in /app/web (if they exist)"
    if [ -d "/app/web" ]; then
        ls -la /app/web
    else
        echo "Web app directory not yet copied to /app/web"
    fi
    
    # Create config template file on first run
    echo "==> First run - creating configuration template"
    
    # Generate random secrets
    JWT_SECRET=$(openssl rand -hex 32)
    SESSION_SECRET=$(openssl rand -hex 32)
    MASTER_KEY=$(openssl rand -hex 32)
    
    # Replace variables in template for things we know
    sed \
        -e "s|%%POSTGRESQL_HOST%%|${CLOUDRON_POSTGRESQL_HOST}|g" \
        -e "s|%%POSTGRESQL_PORT%%|${CLOUDRON_POSTGRESQL_PORT}|g" \
        -e "s|%%POSTGRESQL_USERNAME%%|${CLOUDRON_POSTGRESQL_USERNAME}|g" \
        -e "s|%%POSTGRESQL_PASSWORD%%|${CLOUDRON_POSTGRESQL_PASSWORD}|g" \
        -e "s|%%POSTGRESQL_DATABASE%%|${CLOUDRON_POSTGRESQL_DATABASE}|g" \
        -e "s|%%APP_ORIGIN%%|${CLOUDRON_APP_ORIGIN}|g" \
        -e "s|%%MAIL_SMTP_SERVER%%|${CLOUDRON_MAIL_SMTP_SERVER}|g" \
        -e "s|%%MAIL_SMTP_PORT%%|${CLOUDRON_MAIL_SMTP_PORT}|g" \
        -e "s|%%MAIL_SMTP_USERNAME%%|${CLOUDRON_MAIL_SMTP_USERNAME}|g" \
        -e "s|%%MAIL_SMTP_PASSWORD%%|${CLOUDRON_MAIL_SMTP_PASSWORD}|g" \
        -e "s|%%MAIL_FROM%%|${CLOUDRON_MAIL_FROM}|g" \
        -e "s|%%MAIL_FROM_DISPLAY_NAME%%|${CLOUDRON_MAIL_FROM_DISPLAY_NAME}|g" \
        -e "s|%%JWT_SECRET%%|${JWT_SECRET}|g" \
        -e "s|%%SESSION_SECRET%%|${SESSION_SECRET}|g" \
        -e "s|%%MASTER_KEY%%|${MASTER_KEY}|g" \
        /app/pkg/config.template.yaml > /app/data/config/config.yaml
    
    # Create an S3 configuration file template
    cat > /app/data/config/s3.env.template <<EOT
# S3 Configuration for Ente
# Please copy this file to s3.env and fill in your S3 credentials

# S3 endpoint URL (example: https://s3.amazonaws.com or https://s3.eu-central-2.wasabisys.com)
S3_ENDPOINT=https://your-s3-endpoint

# S3 region (example: us-east-1)
S3_REGION=your-region

# S3 bucket name
S3_BUCKET=your-bucket-name

# S3 access key
S3_ACCESS_KEY=your-access-key

# S3 secret key
S3_SECRET_KEY=your-secret-key

# Optional: prefix for objects within the bucket (example: ente/)
S3_PREFIX=
EOT

    echo "==> IMPORTANT: S3 storage configuration required"
    echo "==> Please configure your S3 storage as follows:"
    echo "1. Log into your Cloudron dashboard"
    echo "2. Go to the app's configuration page"
    echo "3. Edit the file /app/data/config/s3.env"
    echo "4. Restart the app"
    
    # Mark initialization as complete
    touch /app/data/.initialized
    echo "==> Initialization complete"
fi

# Check if configuration exists
if [ ! -f "/app/data/config/s3.env" ]; then
    echo "==> First run - creating configuration template"
    mkdir -p /app/data/config
    
    # Create a template S3 configuration file
    echo "==> Creating S3 configuration template"
    cat > /app/data/config/s3.env.template <<EOT
# S3 Configuration for Ente
# Please copy this file to s3.env and fill in your S3 credentials

# S3 endpoint URL (example: https://s3.amazonaws.com or https://s3.eu-central-2.wasabisys.com)
S3_ENDPOINT=https://your-s3-endpoint

# S3 region (example: us-east-1)
S3_REGION=your-region

# S3 bucket name
S3_BUCKET=your-bucket-name

# S3 access key
S3_ACCESS_KEY=your-access-key

# S3 secret key
S3_SECRET_KEY=your-secret-key

# Optional: prefix for objects within the bucket (example: ente/)
S3_PREFIX=
EOT

    # Create an empty s3.env file to prevent errors
    touch /app/data/config/s3.env
    
    # Display an important notice about S3 configuration
    echo "==> IMPORTANT: S3 storage configuration required"
    echo "==> Please configure your S3 storage as follows:"
    echo "1. Log into your Cloudron dashboard"
    echo "2. Go to the app's configuration page"
    echo "3. Edit the file /app/data/config/s3.env"
    echo "4. Restart the app"
else
    echo "==> Using existing S3 configuration"
fi

# Check if s3.env is empty
if [ ! -s "/app/data/config/s3.env" ]; then
    echo "==> WARNING: S3 configuration file is empty. The app will not function correctly until configured."
    echo "==> Please refer to the template at /app/data/config/s3.env.template for instructions."
fi

# Source S3 configuration
if [ -f /app/data/config/s3.env ]; then
    echo "==> Sourcing S3 configuration from /app/data/config/s3.env"
    source /app/data/config/s3.env
fi

# Display S3 configuration (masking sensitive values)
echo "==> S3 Configuration:"
echo "Endpoint: ${S3_ENDPOINT}"
echo "Region: ${S3_REGION}"
echo "Bucket: ${S3_BUCKET}"
echo "Prefix: ${S3_PREFIX:-}"

# Create museum.yaml for proper S3 configuration
echo "==> Creating museum.yaml configuration"
cat > /app/data/config/museum.yaml <<EOT
s3:
    are_local_buckets: false
    use_path_style_urls: true
    s3-storage:
        key: ${S3_ACCESS_KEY}
        secret: ${S3_SECRET_KEY}
        endpoint: ${S3_ENDPOINT}
        region: ${S3_REGION}
        bucket: ${S3_BUCKET}
EOT
echo "==> Created museum.yaml with S3 configuration"

# Update the config file with S3 credentials
sed -i \
    -e "s|%%S3_ENDPOINT%%|${S3_ENDPOINT}|g" \
    -e "s|%%S3_REGION%%|${S3_REGION}|g" \
    -e "s|%%S3_BUCKET%%|${S3_BUCKET}|g" \
    -e "s|%%S3_ACCESS_KEY%%|${S3_ACCESS_KEY}|g" \
    -e "s|%%S3_SECRET_KEY%%|${S3_SECRET_KEY}|g" \
    -e "s|%%S3_PREFIX%%|${S3_PREFIX:-}|g" \
    /app/data/config/config.yaml

# Set storage type to S3 in config
sed -i 's|storage.type: "local"|storage.type: "s3"|g' /app/data/config/config.yaml
sed -i 's|s3.are_local_buckets: true|s3.are_local_buckets: false|g' /app/data/config/config.yaml

# Install or verify required packages
echo "==> Checking for required packages"
if ! command -v caddy &> /dev/null; then
    echo "==> Installing Caddy"
    apt-get update && apt-get install -y debian-keyring debian-archive-keyring apt-transport-https
    curl -1sLf 'https://dl.cloudsmith.io/public/caddy/stable/gpg.key' | gpg --dearmor -o /usr/share/keyrings/caddy-stable-archive-keyring.gpg
    curl -1sLf 'https://dl.cloudsmith.io/public/caddy/stable/debian.deb.txt' | tee /etc/apt/sources.list.d/caddy-stable.list
    apt-get update && apt-get install -y caddy
fi

# Set up the API endpoint for the web apps
API_ENDPOINT="${CLOUDRON_APP_ORIGIN}/api"
echo "==> Setting API endpoint to $API_ENDPOINT"

# Set environment variables for the web apps
export ENTE_API_ENDPOINT=$API_ENDPOINT
export NEXT_PUBLIC_ENTE_ENDPOINT=$API_ENDPOINT
export REACT_APP_ENTE_ENDPOINT=$API_ENDPOINT
export VUE_APP_ENTE_ENDPOINT=$API_ENDPOINT
echo "==> Set environment variables for web apps"

# Create directory for configuration files
mkdir -p /app/data/public
mkdir -p /app/data/scripts

# Create a debugging script
cat > /app/data/public/debug.js <<EOT
// Debugging script for Ente
(function() {
  console.log("Debug script loaded");
  
  // Intercept URL constructor
  const originalURL = window.URL;
  window.URL = function(url, base) {
    console.log("URL constructor called with:", url, base);
    try {
      return new originalURL(url, base);
    } catch (e) {
      console.error("URL construction failed:", e.message);
      console.error("URL:", url);
      console.error("Base:", base);
      console.error("Stack:", e.stack);
      
      // Try to fix common issues
      if (url && !url.startsWith("http") && !url.startsWith("/")) {
        console.log("Attempting to fix relative URL by adding leading slash");
        return new originalURL("/" + url, base);
      }
      throw e;
    }
  };
  
  // Create debug overlay
  const debugDiv = document.createElement('div');
  debugDiv.style.position = 'fixed';
  debugDiv.style.bottom = '10px';
  debugDiv.style.right = '10px';
  debugDiv.style.backgroundColor = 'rgba(0,0,0,0.7)';
  debugDiv.style.color = 'white';
  debugDiv.style.padding = '10px';
  debugDiv.style.borderRadius = '5px';
  debugDiv.style.zIndex = '9999';
  debugDiv.style.maxWidth = '400px';
  debugDiv.style.maxHeight = '200px';
  debugDiv.style.overflow = 'auto';
  debugDiv.innerHTML = '<h3>Ente Debug Info</h3>';
  
  // Add configuration info
  const configInfo = document.createElement('div');
  configInfo.innerHTML = 'ENTE_CONFIG: ' + JSON.stringify(window.ENTE_CONFIG || {}) + '<br>' +
    'process.env.NEXT_PUBLIC_ENTE_ENDPOINT: ' + (window.process?.env?.NEXT_PUBLIC_ENTE_ENDPOINT || 'undefined') + '<br>' +
    'localStorage ENTE_CONFIG: ' + localStorage.getItem('ENTE_CONFIG') + '<br>' +
    'localStorage NEXT_PUBLIC_ENTE_ENDPOINT: ' + localStorage.getItem('NEXT_PUBLIC_ENTE_ENDPOINT');
  debugDiv.appendChild(configInfo);
  
  // Add toggle button
  const toggleButton = document.createElement('button');
  toggleButton.innerText = 'Toggle Debug Info';
  toggleButton.style.marginTop = '10px';
  toggleButton.onclick = function() {
    configInfo.style.display = configInfo.style.display === 'none' ? 'block' : 'none';
  };
  debugDiv.appendChild(toggleButton);
  
  // Add to document when it's ready
  if (document.body) {
    document.body.appendChild(debugDiv);
  } else {
    window.addEventListener('DOMContentLoaded', function() {
      document.body.appendChild(debugDiv);
    });
  }
})();
EOT

# Create a configuration script
cat > /app/data/public/config.js <<EOT
// Direct configuration for Ente
window.ENTE_CONFIG = {
  API_URL: "${API_ENDPOINT}"
};

// Next.js environment variables
window.process = window.process || {};
window.process.env = window.process.env || {};
window.process.env.NEXT_PUBLIC_ENTE_ENDPOINT = "${API_ENDPOINT}";
window.process.env.REACT_APP_ENTE_ENDPOINT = "${API_ENDPOINT}";
window.process.env.VUE_APP_ENTE_ENDPOINT = "${API_ENDPOINT}";

// Store in localStorage for persistence
try {
  localStorage.setItem('ENTE_CONFIG', JSON.stringify(window.ENTE_CONFIG));
  localStorage.setItem('NEXT_PUBLIC_ENTE_ENDPOINT', "${API_ENDPOINT}");
} catch (e) {
  console.error("Failed to store config in localStorage:", e);
}

console.log("Ente config loaded - API_URL:", window.ENTE_CONFIG.API_URL);
EOT

# Create debug info HTML page
cat > /app/data/public/debug.html <<EOT
<!DOCTYPE html>
<html>
<head>
  <meta charset="utf-8">
  <title>Ente Debug Info</title>
  <style>
    body { font-family: Arial, sans-serif; margin: 20px; }
    .debug-section { margin-bottom: 20px; padding: 10px; border: 1px solid #ccc; }
    h1 { color: #333; }
    pre { background-color: #f5f5f5; padding: 10px; border-radius: 5px; overflow-x: auto; }
  </style>
</head>
<body>
  <h1>Ente Debug Information</h1>
  
  <div class="debug-section">
    <h2>Frontend Configuration</h2>
    <pre id="config-info">Loading...</pre>
  </div>
  
  <div class="debug-section">
    <h2>URL Test</h2>
    <p>Testing URL construction with API endpoint:</p>
    <pre id="url-test">Running test...</pre>
  </div>
  
  <div class="debug-section">
    <h2>API Health Check</h2>
    <pre id="api-health">Checking API health...</pre>
  </div>
  
  <script>
    // Define configuration globally
    window.ENTE_CONFIG = {
      API_URL: "${API_ENDPOINT}"
    };
    
    // Set environment variables for Next.js apps
    window.process = window.process || {};
    window.process.env = window.process.env || {};
    window.process.env.NEXT_PUBLIC_ENTE_ENDPOINT = "${API_ENDPOINT}";
    
    // Store in localStorage
    localStorage.setItem('ENTE_CONFIG', JSON.stringify(window.ENTE_CONFIG));
    localStorage.setItem('NEXT_PUBLIC_ENTE_ENDPOINT', "${API_ENDPOINT}");
    
    // Display configuration
    document.getElementById('config-info').textContent = 
      'window.ENTE_CONFIG = ' + JSON.stringify(window.ENTE_CONFIG, null, 2) + '\n' +
      'window.process.env.NEXT_PUBLIC_ENTE_ENDPOINT = ' + window.process.env.NEXT_PUBLIC_ENTE_ENDPOINT + '\n' +
      'localStorage[\'ENTE_CONFIG\'] = ' + localStorage.getItem('ENTE_CONFIG') + '\n' +
      'localStorage[\'NEXT_PUBLIC_ENTE_ENDPOINT\'] = ' + localStorage.getItem('NEXT_PUBLIC_ENTE_ENDPOINT');
    
    // Test URL construction
    try {
      const apiUrl = window.ENTE_CONFIG.API_URL;
      const testUrl = new URL('/users/ott', apiUrl);
      document.getElementById('url-test').textContent = 
        'API URL: ' + apiUrl + '\n' +
        'Test URL (/users/ott): ' + testUrl.toString() + '\n' +
        'Result: SUCCESS';
    } catch (e) {
      document.getElementById('url-test').textContent = 
        'Error: ' + e.message + '\n' +
        'Stack: ' + e.stack;
    }
    
    // Test API health
    fetch('/api/health')
      .then(response => {
        if (response.ok) return response.text();
        throw new Error('API returned status: ' + response.status);
      })
      .then(data => {
        document.getElementById('api-health').textContent = 'API health check: OK\nResponse: ' + data;
      })
      .catch(err => {
        document.getElementById('api-health').textContent = 'API health check failed: ' + err.message;
      });
  </script>
</body>
</html>
EOT

# Create a simple app that loads the config and redirects to the main app
cat > /app/data/public/index.html <<EOT
<!DOCTYPE html>
<html>
<head>
  <meta charset="utf-8">
  <title>Ente</title>
  <script src="/config.js"></script>
  <script src="/debug.js"></script>
  <script>
    // Redirect to the photos app after a brief delay
    setTimeout(function() {
      window.location.href = "/photos/";
    }, 300);
  </script>
</head>
<body>
  <h1>Loading Ente...</h1>
  <p>You will be redirected automatically.</p>
  <p><a href="/debug">Debug Information</a></p>
</body>
</html>
EOT

# Set up Caddy
echo "==> Setting up Caddy server"
mkdir -p /app/data/caddy
chmod -R 777 /app/data/caddy

# Define ports
CADDY_PORT=3080
API_PORT=8080

# Check if ports are available
echo "==> Checking port availability"
if lsof -i:$CADDY_PORT > /dev/null 2>&1; then
    echo "==> WARNING: Port $CADDY_PORT is already in use"
else
    echo "==> Port $CADDY_PORT is available for Caddy"
fi

if lsof -i:$API_PORT > /dev/null 2>&1; then
    echo "==> WARNING: Port $API_PORT is already in use"
else
    echo "==> Port $API_PORT is available for API server"
fi

# Create the Caddyfile
cat > /app/data/caddy/Caddyfile <<EOT
{
    admin off
    auto_https off
    log {
        output file /app/data/caddy/caddy.log {
            roll_size 10MB
            roll_keep 10
        }
    }
}

:$CADDY_PORT {
    log {
        output file /app/data/caddy/access.log {
            roll_size 10MB
            roll_keep 10
        }
    }
    
    # Headers for WebAssembly and IndexedDB
    header {
        Cross-Origin-Embedder-Policy "credentialless"
        Cross-Origin-Opener-Policy "same-origin"
        Cross-Origin-Resource-Policy "cross-origin"
    }
    
    # Serve configuration scripts
    handle /config.js {
        root * /app/data/public
        file_server
    }
    
    handle /debug.js {
        root * /app/data/public
        file_server
    }
    
    # Serve debug page
    handle /debug {
        root * /app/data/public
        rewrite * /debug.html
        file_server
    }
    
    # Health check endpoints
    handle /health {
        respond "OK" 200
    }
    
    handle /healthcheck {
        respond "OK" 200
    }
    
    # API health check endpoint - direct proxy to API
    handle /api/health {
        reverse_proxy localhost:$API_PORT
    }
    
    # Root handler
    handle / {
        root * /app/data/public
        file_server
    }
    
    # Server web apps
    handle /photos* {
        uri replace /photos /
        root * /app/web/photos
        
        # Inject config script
        handle_path /* {
            @html {
                path *.html index.html
                not path */assets/* */static/* */img/* */css/* */js/*
            }
            
            header @html Content-Type "text/html; charset=utf-8"
            
            # Use rewrite to modify HTML content
            @isIndex path /index.html
            rewrite @isIndex /photos-injected.html
            
            file_server
        }
    }
    
    handle /accounts* {
        uri replace /accounts /
        root * /app/web/accounts
        
        handle_path /* {
            @html {
                path *.html index.html
                not path */assets/* */static/* */img/* */css/* */js/*
            }
            
            header @html Content-Type "text/html; charset=utf-8"
            
            # Use rewrite to modify HTML content
            @isIndex path /index.html
            rewrite @isIndex /accounts-injected.html
            
            file_server
        }
    }
    
    handle /auth* {
        uri replace /auth /
        root * /app/web/auth
        
        handle_path /* {
            @html {
                path *.html index.html
                not path */assets/* */static/* */img/* */css/* */js/*
            }
            
            header @html Content-Type "text/html; charset=utf-8"
            
            # Use rewrite to modify HTML content
            @isIndex path /index.html
            rewrite @isIndex /auth-injected.html
            
            file_server
        }
    }
    
    handle /cast* {
        uri replace /cast /
        root * /app/web/cast
        
        handle_path /* {
            @html {
                path *.html index.html
                not path */assets/* */static/* */img/* */css/* */js/*
            }
            
            header @html Content-Type "text/html; charset=utf-8"
            
            # Use rewrite to modify HTML content
            @isIndex path /index.html
            rewrite @isIndex /cast-injected.html
            
            file_server
        }
    }
    
    # Proxy API calls to the backend server
    handle /api* {
        reverse_proxy localhost:$API_PORT
    }
}
EOT

echo "==> Created Caddyfile at /app/data/caddy/Caddyfile"

# Create injected HTML files for each app
for app_name in "photos" "accounts" "auth" "cast"; do
    # Create an injected HTML file that loads the config
    cat > "/app/data/public/${app_name}-injected.html" <<EOT
<!DOCTYPE html>
<html>
<head>
  <meta charset="utf-8">
  <title>Ente ${app_name^}</title>
  <script src="/config.js"></script>
  <script>
    // Redirect to the actual app after loading config
    window.onload = function() {
      const iframe = document.createElement('iframe');
      iframe.style.width = '100%';
      iframe.style.height = '100%';
      iframe.style.border = 'none';
      iframe.style.position = 'absolute';
      iframe.style.top = '0';
      iframe.style.left = '0';
      iframe.src = "/${app_name}/index.html";
      document.body.appendChild(iframe);
    };
  </script>
  <style>
    body {
      margin: 0;
      padding: 0;
      height: 100vh;
      width: 100vw;
      overflow: hidden;
    }
  </style>
</head>
<body>
  <div id="loading" style="display: flex; justify-content: center; align-items: center; height: 100vh; flex-direction: column;">
    <h1>Loading Ente ${app_name^}...</h1>
    <p>Please wait while the application initializes.</p>
  </div>
</body>
</html>
EOT
done

echo "==> Created injected HTML files for all apps"

# Start Caddy
echo "==> Starting Caddy on port $CADDY_PORT"
caddy run --config /app/data/caddy/Caddyfile --adapter caddyfile &
CADDY_PID=$!
echo "==> Caddy started with PID $CADDY_PID"

# Wait for Caddy to start
sleep 2

# Test Caddy connectivity
echo "==> Testing Caddy connectivity"
for i in {1..5}; do
    if curl -s --max-time 2 --head --fail http://localhost:$CADDY_PORT/health > /dev/null; then
        echo "==> Caddy is running properly on port $CADDY_PORT"
        break
    else
        if [ $i -eq 5 ]; then
            echo "==> Failed to connect to Caddy after multiple attempts"
            echo "==> Last 20 lines of Caddy log:"
            tail -20 /app/data/caddy/caddy.log || echo "==> No Caddy log available"
            echo "==> Network ports in use:"
            netstat -tuln || echo "==> netstat command not available"
        else
            echo "==> Attempt $i: Waiting for Caddy to start... (1 second)"
            sleep 1
        fi
    fi
done

# Determine available memory and set limits accordingly
if [[ -f /sys/fs/cgroup/cgroup.controllers ]]; then # cgroup v2
    memory_limit=$(cat /sys/fs/cgroup/memory.max)
    if [[ "$memory_limit" != "max" ]]; then
        MEMORY_MB=$((memory_limit / 1024 / 1024))
    else
        MEMORY_MB=$(free -m | awk '/^Mem:/{print $2}')
    fi
else # cgroup v1
    if [ -f /sys/fs/cgroup/memory/memory.limit_in_bytes ]; then
        memory_limit=$(cat /sys/fs/cgroup/memory/memory.limit_in_bytes)
        MEMORY_MB=$((memory_limit / 1024 / 1024))
    else
        MEMORY_MB=$(free -m | awk '/^Mem:/{print $2}')
    fi
fi

echo "==> Available memory: ${MEMORY_MB}MB"

# Test database connectivity
echo "==> Checking database connectivity"
PGPASSWORD="${CLOUDRON_POSTGRESQL_PASSWORD}" psql -h ${CLOUDRON_POSTGRESQL_HOST} -p ${CLOUDRON_POSTGRESQL_PORT} -U ${CLOUDRON_POSTGRESQL_USERNAME} -d ${CLOUDRON_POSTGRESQL_DATABASE} -c "SELECT 1" > /dev/null

if [ $? -ne 0 ]; then
    echo "==> ERROR: Failed to connect to database"
    echo "Host: ${CLOUDRON_POSTGRESQL_HOST}"
    echo "Port: ${CLOUDRON_POSTGRESQL_PORT}"
    echo "User: ${CLOUDRON_POSTGRESQL_USERNAME}"
    echo "Database: ${CLOUDRON_POSTGRESQL_DATABASE}"
    exit 1
fi
echo "==> Successfully connected to database"

# Create proper Go module environment
echo "==> Setting up Go module environment"
if [ -f "$SERVER_DIR/go.mod" ]; then
    echo "==> Found go.mod in $SERVER_DIR"
    mkdir -p /app/data/go
    cp "$SERVER_DIR/go.mod" "/app/data/go/go.mod"
    if [ -f "$SERVER_DIR/go.sum" ]; then
        cp "$SERVER_DIR/go.sum" "/app/data/go/go.sum"
    fi
    echo "==> Copied go.mod to /app/data/go/go.mod"
else
    echo "==> WARNING: No go.mod found in $SERVER_DIR"
    # Create a minimal go.mod file
    mkdir -p /app/data/go
    cat > /app/data/go/go.mod <<EOT
module ente.io/museum

go 1.24
EOT
    echo "==> Created minimal go.mod in /app/data/go/go.mod"
fi
# Ensure the right permissions
chmod 644 /app/data/go/go.mod
# Setup Go directories with proper permissions
mkdir -p /app/data/go/pkg/mod /app/data/go/cache
chmod -R 777 /app/data/go
chown -R cloudron:cloudron /app/data/go

# Fix database migration state if needed
echo "==> Checking database migration state"
if [ -d "$SERVER_DIR/cmd/museum" ]; then
    echo "==> Attempting to fix dirty migration state"
    cd "$SERVER_DIR"
    
    # Create migrations log directory
    mkdir -p /app/data/logs/migrations
    
    # Set Go environment variables
    export GOMODCACHE="/app/data/go/pkg/mod"
    export GOCACHE="/app/data/go/cache"
    export GO111MODULE=on
    export GOFLAGS="-modfile=/app/data/go/go.mod -mod=mod"
    
    echo "==> Forcing migration version to 25"
    if /usr/local/bin/gosu cloudron:cloudron env \
        GOCACHE="$GOCACHE" \
        GOMODCACHE="$GOMODCACHE" \
        GO111MODULE=on \
        GOFLAGS="$GOFLAGS" \
        cd "$SERVER_DIR" && go run cmd/museum/main.go db force 25 > /app/data/logs/migrations/force.log 2>&1; then
        echo "==> Successfully forced migration version"
    else
        echo "==> WARNING: Could not force migration version"
        echo "==> Migration force log:"
        cat /app/data/logs/migrations/force.log
    fi
else
    echo "==> Skipping migration state check: cmd/museum not found"
fi

# Start the Museum server with proper environment variables
echo "==> Starting Museum server"
cd "$SERVER_DIR"

# Set necessary environment variables
export MUSEUM_CONFIG="/app/data/config/museum.yaml"
export MUSEUM_DB_HOST="${CLOUDRON_POSTGRESQL_HOST}"
export MUSEUM_DB_PORT="${CLOUDRON_POSTGRESQL_PORT}"
export MUSEUM_DB_USER="${CLOUDRON_POSTGRESQL_USERNAME}"
export MUSEUM_DB_PASSWORD="${CLOUDRON_POSTGRESQL_PASSWORD}"
export MUSEUM_DB_NAME="${CLOUDRON_POSTGRESQL_DATABASE}"
export ENTE_LOG_LEVEL=debug
export GOMODCACHE="/app/data/go/pkg/mod"
export GOCACHE="/app/data/go/cache"
export GO111MODULE=on
export GOFLAGS="-modfile=/app/data/go/go.mod -mod=mod"

# Check if there's a pre-built binary
MUSEUM_BIN=""
if [ -f "$SERVER_DIR/bin/museum" ] && [ -x "$SERVER_DIR/bin/museum" ]; then
    echo "==> Found Museum binary at $SERVER_DIR/bin/museum"
    MUSEUM_BIN="$SERVER_DIR/bin/museum"
elif [ -f "/app/data/go/bin/museum" ] && [ -x "/app/data/go/bin/museum" ]; then
    echo "==> Found Museum binary at /app/data/go/bin/museum"
    MUSEUM_BIN="/app/data/go/bin/museum"
fi

# Start server
if [ -n "$MUSEUM_BIN" ]; then
    echo "==> Starting Museum from binary: $MUSEUM_BIN"
    $MUSEUM_BIN serve > /app/data/logs/museum.log 2>&1 &
    SERVER_PID=$!
elif [ -d "$SERVER_DIR/cmd/museum" ]; then
    echo "==> Starting Museum from source"
    /usr/local/bin/gosu cloudron:cloudron env \
        GOCACHE="$GOCACHE" \
        GOMODCACHE="$GOMODCACHE" \
        GO111MODULE=on \
        GOFLAGS="$GOFLAGS" \
        cd "$SERVER_DIR" && go run cmd/museum/main.go serve > /app/data/logs/museum.log 2>&1 &
    SERVER_PID=$!
else
    echo "==> ERROR: Museum server not found"
    echo "==> Starting a mock server"
    
    # Create a temporary directory for a simple Go server
    mkdir -p /tmp/mock-server
    cat > /tmp/mock-server/main.go <<EOT
package main

import (
    "fmt"
    "log"
    "net/http"
)

func main() {
    // Add a health endpoint
    http.HandleFunc("/health", func(w http.ResponseWriter, r *http.Request) {
        w.Header().Set("Content-Type", "application/json")
        fmt.Fprint(w, `{"status":"ok","message":"Mock server running"}`)
    })
    
    // Handle all other requests
    http.HandleFunc("/", func(w http.ResponseWriter, r *http.Request) {
        w.Header().Set("Content-Type", "application/json")
        fmt.Fprintf(w, `{"status":"mock","endpoint":"%s","method":"%s"}`, r.URL.Path, r.Method)
    })
    
    // Start the server
    log.Printf("Starting mock server on port %d\n", 8080)
    
    if err := http.ListenAndServe(":8080", nil); err != nil {
        log.Fatalf("Failed to start server: %v", err)
    }
}
EOT
    
    # Run the mock server
    cd /tmp/mock-server
    go run main.go > /app/data/logs/museum.log 2>&1 &
    SERVER_PID=$!
    
    echo "==> Mock server started with PID $SERVER_PID"
fi

echo "==> Server started with PID $SERVER_PID"

# Test if API is responding
echo "==> Testing API connectivity"
for i in {1..5}; do
    if curl -s --max-time 2 --fail http://localhost:$API_PORT/health > /dev/null; then
        echo "==> API is responding on port $API_PORT"
        break
    else
        if [ $i -eq 5 ]; then
            echo "==> WARNING: API is not responding after several attempts"
            echo "==> Last 20 lines of museum.log:"
            tail -20 /app/data/logs/museum.log || echo "==> No museum.log available"
        else
            echo "==> Attempt $i: Waiting for API to start... (2 seconds)"
            sleep 2
        fi
    fi
done

echo "==> Application is now running"
echo "==> Access your Ente instance at: $CLOUDRON_APP_ORIGIN"
echo "==> To view debug information, visit: $CLOUDRON_APP_ORIGIN/debug"

echo "==> Entering wait state - press Ctrl+C to stop"
# Wait for all background processes to complete (or for user to interrupt)
wait $SERVER_PID
wait $CADDY_PID 